class GeometricPolygon < Struct.new(:vertices)

  def bounding_box
    leftbottom = GeometricPoint.new(vertices.map(&:x).min, vertices.map(&:y).min)
    righttop = GeometricPoint.new(vertices.map(&:x).max, vertices.map(&:y).max)

    GeometricBoundingBox.new(leftbottom, righttop)
  end

  def area
    sum = 0.0
    (0..vertices.length-1).each do |i|
      a = vertices[i-1]
      b = vertices[i]

      sum = sum + ((a.x * b.y) - (a.y * b.x))
    end
    (sum / 2).abs
  end

  def contains?(point)
    point_in_polygon = GeometricPointInPolygon.new(point, self)
    point_in_polygon.inside? || point_in_polygon.on_the_boundary?
  end

  def edges
    edges = []

    1.upto(vertices.length - 1) do |vertex_index|
      edges << GeometricSegment.new(vertices[vertex_index - 1], vertices[vertex_index])
    end

    edges << GeometricSegment.new(vertices.last, vertices.first)
  end

end